
/* nxlcau4h.c				*/
/* (4,1/2) Linear Cellular Automaton	*/

/* Reference:				*/
/*					*/
/*    Kenneth E. Perry			*/
/*    Abstract Mathematical Art		*/
/*    BYTE				*/
/*    December, 1986			*/
/*    pages 181-192			*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1989		*/
/*    Copyright (C) 1990		*/
/*    Harold V. McIntosh		*/
/*    Gerardo Cisneros S.		*/

/* G. Cisneros, 4.3.87						*/
/* 28 April 1987 - version for XVI Feria de Puebla [HVM]	*/
/* 12 June 1987 - cartesian product of (2,1) rules [HVM]	*/
/* 12 June 1987 - (2,1) rule with memory  [HVM]			*/
/* 14 June 1987 - individual cycles of evolution [HVM]		*/
/* 17 June 1987 - p-adic representation in plane [HVM]		*/
/* 22 June 1987 - 2 speed gliders via 16 transitions [HVM]	*/
/* 26 June 1987 - push, pop the rule [HVM]			*/
/* 26 June 1987 - conserve position of rule cursor [HVM]	*/
/* 27 June 1987 - incremental rule construction [HVM]		*/
/* 29 June 1987 - conserve position of cell pointer [HVM]	*/
/* 30 June 1987 - mark & unmark transitions, xchg x&X [HVM]	*/
/* 25 July 1987 - display and edit de Bruijn diagrams [HVM]	*/
/* 27 July 1987 - graph of transition probabilities [HVM]	*/
/* 4 September 1987 - PROB41.C for option 't' [HVM]		*/
/* 21 October 1987 - program disks disappeared			*/
/* 24 December 1987 - program reconstructed from listings	*/
/* 20 February 1988 - RIJN31.C for option 'd' [HVM]		*/
/* 10 April 1988 - include rule engineering from LCA41 [HVM]	*/
/* 11 March 1989 - option 'x' in trule [HVM]			*/
/* 17 September 1989 - directory for demonstration rules [HVM]	*/
/* 5 October 1989 - modified for radius 1/2 [HVM]		*/
/* 10 February 1990 - Garden of Eden option [HVM] 		*/

// # include <bdos.h>
# include "nxlcau4h.h"
# include "rec.h"
# define LROW		 9	/* top row of line panel */
# define DROWS		20	/* rows in demo menu	 */
# define DCOLS		32	/* second col in demo m. */
# define NX		20	/* number of sample rules */
//# define XLEN    	128     /*  buffer length for rec predefinided */

char ixrule[NX][DS]= {

	"0002111311130002",	/*  1                       */
	"0001110110111010",	/*  2                       */
	"0003003203213211",	/*  3 - eventual glider gun */
	"0012012312322320",	/*  4 - class iv            */
	"0012012312322320",	/*  5 - class iv            */
	"0021021321311310",	/*  6 - glider implosion    */
	"0033132213021100",	/*  7 - evolutionary #1     */
	"0100112122230333",	/*  8 - Fisch's eater       */
	"0120120320330333",	/*  9 - totalized eater     */
	"0123112322230333",	/* 10 - majority            */
	"0122100210111010",	/* 11                       */
	"0122210020111010",	/* 12                       */
	"0201001220111010",	/* 13                       */
	"0212002010111010",	/* 14                       */
	"0213021320132013",	/* 15 - reversible rule     */
	"0213021320312031",	/* 16 - big blotches        */
	"1323231031302012",	/* 17 - class iv on bgrnd   */
	"2120132302030131",	/* 18 - eater variant       */
	"3102133033213111",	/* 19 - pepper and salt     */
	"3311203102203302"	/* 20 - evolutionary #2     */

	};
/****************** modify added *************************/

char   gf4rule[13];		/* coefficients of gf4 rule	*/
char   strule[KK][SS];
char   ttrule[TS];
char   tarule[TS];

/****************** modify REC *************************/


Inst xprg[10][XLEN]; 
char recrule[NY][CLEN]={
	"(\"18DFA439\" W;  [ 1 - rule  ])",
	"(\"90DA76F4\" W;  [ 2 - rule  ])",
	"(\"43DA215F\" W;  [ 3 - rule  ])",
	"(\"150D43A2\" W;  [ 4 - rule  ])",
	"(\"30B56C2A\" W;  [ 5 - rule  ])",
	"(\"7321F5AD\" W;  [ 6 - rule  ])",
	"(\"101FD217\" W;  [ 7 - rule  ])",
	"(\"191BCAF4\" W;  [ 8 - rule  ])"
	};



/*********************************************************/
/*** - - - - - R E G L A S   R E V E R S I B L E S - - - - - - - ***/ 
char reverules[NRR][40]={
			"(\"0055AAFF\"W; [ 1 - ])",
			"(\"0055AFFA\"W; [ 2 - ])",
			"(\"0055BAEF\"W; [ 3 - ])",
			"(\"0056E9BF\"W; [ 4 - ])",
			"(\"0066BFD9\"W; [ 5 - ])",
			"(\"006ABFD5\"W; [ 6 - ])",
			"(\"0154AFFA\"W; [ 7 - ])",
			"(\"0550AFFA\"W; [ 8 - ])",
			"(\"05AF05AF\"W; [ 9 - ])",
			"(\"05AF05EB\"W; [10 - ])",
			"(\"05AF0FA5\"W; [11 - ])"
		       };

parneih    amat;
parneih    bmat;
parneih    cmat;
parneih    dmat;
parneih    fmat;
parneih    gmat;
parneih    hmat;
parneih    xmat;

twoparn    smat;
twoparn    tmat;
twoparn    umat;
twoparn    vmat;

double stt, sta[5][K3+1];
double blpr[KK];			/* probability from blocks	*/

char   tabl[20][SL];			/* workspace to construct table */
char   ascrule[KK][KK];			/* ASCII transition values      */
char   auxrule[KK][KK];			/* auxiliary transition table   */
char   rulstk[10][KK][KK];		/* pushdown for rules 		*/
char   prule1[8], prule2[8];		/* product rule specification 	*/
char   evrule[2][2][2][2];		/* evolutionary rule 		*/
char   bhrul1[2][2], bhrul2[2][3];	/* binary r=1/2 rule #1,2 	*/
int    binrule[KK][KK];
char   arule[DS], arr1[AL], arr2[AL], arr3[AL];
int    ru, ru0, ru1;					/* rule cursor */
int    li, lj;						/* cell pointer */
int    di, dj;						/* demo pointer */
int    rulptr;						/* rule pd ptr */
int    sc;						/* screen colors */
int    ix0, iy0;					/* origin for pen moves */
double wmul[KK], wmvl[KK];				/* left mass point */
double wmur[KK], wmvr[KK];				/* right mass point */ char idef[1][30] = { "(z(!100!0r:;)1r2r3r(0r:;);a"};
lcau4hmain() {
int  i, j, i0, i1, i2, i3;
int  more = 'r';
char a, b, c;

for (i=0; i<TS; i++) ttrule[i]='0';

ru=6; ru0=0; ru1=0;
li=SL/2; lj=0;						/* line editor cursor */
di=0; dj=1;						/* demonstration cursor */
sc=0;
rulptr=0;

    videopalette(WHCYMAG);				/* white/cyan/magenta */

    tuto();
    while (!kbdst()) rand();				/* wait for keypress */
    kbdin();						/* ignore it */
    videomode(T80X25);
    videoscroll(3,0,5,71,0,3);				/* menu on blue background */
    videoscroll(19,0,24,71,0,3);
    i=(rand()%NX);
    di=i%DROWS;
    dj=(i/DROWS==1?DCOLS:1);
    xtoasc(i);
    ranlin();						/* random initial array */
    auxblnk();						/* uncomitted transitions */

    while (more!='n') {					/* execute multiple runs */
    rmenu();
    lmenu();
    while (0<1) {					/* set up one run */
    c=kbdin();
    if (c=='g') break;					/* go draw graph */
    if (c=='q') more='n';				/* quit for good */
    if (more=='n') break;
    switch (c) {
        case '!': sc=7; break;
	case '*': sc=2; break;				/* white on red */
	case '+': sc=1; break;				/* white screen */
	case '-': sc=0; break;				/* blue screen */
	case '@':					/* numbered tot rule */
	    nutoto(numin(0));
	    totoasc();
	    rmenu();
	    videocursor(0,4,0);
	    break;
	case '$':					/* dozen totalistics */
	    j=numin(0);
	    for (i=0; i<12; i++) {
	      nutoto(j+i);
	      totoasc();
	      ranlin();
	      evolve();
	      }
	    videomode(T80X25);
	    rmenu();
	    lmenu();
	    break;
	case '.':					/* one cycle of evolution */
	    asctobin();
	    onegen(AL);
	    clmenu();
	    break;
	case ',':					/* one cycle of evolution */
	    videomode(COLGRAF);
	    pgrid();
	    for (i=0; i<200; i++) {pprob(); onegen(AL);}
	    videodot(190,195,3);
	    kbdin();
	    videomode(T80X25);
	    rmenu();
	    lmenu();
	    break;
	case 'T':					/* totalistic rule */
	    xblnk();
	    tmenu();
	    edtrule();
	    totoasc(); 
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	      ascrule[i0][i1]=ttrule[i0+i1];
	    videocursor(0,4,0);
	    rmenu();
	    xmenu(totonu(0));
	    break;
	case 't':					/* edit tri prob plots */
	    edtri();
	    rmenu();
	    lmenu();
	    break;
        case 'r':					/* edit rule */	
	    xblnk();
	    edrule();
	    videocursor(0,4,0);
	    rmenu();
	    break;
        case 'l':					/* edit cell string */
	    xblnk();
	    edline(8,40);
	    videocursor(0,3,0);
	    lmenu();
	    break;
	case '\273':					/* F1 */
        case '#':					/* read stored rule */
	    xtoasc(lim(1,dmenu(),NX)-1);
	    rmenu();
	    lmenu();
	    break;
	case 'D':					/* run through samples */
	    for (i=0; i<NX; i++) {
	      xmenu(i);
	      xtoasc(i);
	      ranlin();
	      evolve();
	      }
	    videomode(T80X25);
	    rmenu();
	    break;
	case 'd':					/* de Bruijn diagram */
	    edijn();
	    rmenu();
	    lmenu();
	    break;
	case 'e':					/* Garden of Eden */
	    edard();
	    rmenu();
	    lmenu();
	    break;
	case 'p':					/* product of two (2,1) rules */
	    nutowo1(numin(0));
	    nutowo2(numin(0));
	    for (i0=0; i0<2; i0++)
	    for (i1=0; i1<2; i1++)
	    for (i2=0; i2<2; i2++)
	    for (i3=0; i3<2; i3++)
	    ascrule[2*i0+i1][2*i2+i3]=
		'0'+2*(prule1[2*i0+i2]-'0')+(prule2[2*i1+i3]-'0');
	    xblnk();
	    rmenu();
	    break;
	case 'M':					/* (2,1) rules with memory */
	    nutowo1(numin(0));
	    for (i0=0; i0<2; i0++)
	    for (i1=0; i1<2; i1++)
	    for (i2=0; i2<2; i2++)
	    for (i3=0; i3<2; i3++)
	    ascrule[2*i0+i1][2*i2+i3]='0'+((prule1[2*i1+i3]-'0')==i0?0:1)+2*i1;
	    xblnk();
	    rmenu();
	    break;
	case 'm':					/* (2,1) rules with memory */
	    nutowo1(numin(0));
	    for (i0=0; i0<2; i0++)
	    for (i1=0; i1<2; i1++)
	    for (i2=0; i2<2; i2++)
	    for (i3=0; i3<2; i3++)
	    ascrule[2*i0+i1][2*i2+i3]='0'+2*((prule1[2*i0+i2]-'0')==i1?0:1)+i0;
	    xblnk();
	    rmenu();
	    break;
	case 'N':					/* (2,1) rules with memory */
	    nutowo1(numin(0));
	    for (i0=0; i0<2; i0++)
	    for (i1=0; i1<2; i1++)
	    for (i2=0; i2<2; i2++)
	    for (i3=0; i3<2; i3++)
	    ascrule[2*i0+i1][2*i2+i3]='0'+((prule1[2*i1+i3]-'0')==i2?0:1)+2*i3;
	    xblnk();
	    rmenu();
	    break;
	case 'n':					/* (2,1) rules with memory */
	    nutowo1(numin(0));
	    for (i0=0; i0<2; i0++)
	    for (i1=0; i1<2; i1++)
	    for (i2=0; i2<2; i2++)
	    for (i3=0; i3<2; i3++)
	    ascrule[2*i0+i1][2*i2+i3]='0'+2*((prule1[2*i0+i2]-'0')==i3?0:1)+i2;
	    xblnk();
	    rmenu();
	    break;
	case 'v':					/* evolutionary rule */
	    nutoevo(numin(0));
	    nutobh1(numin(0));
	    nutobh2(numin(0));
	    for (i0=0; i0<2; i0++)
	    for (i1=0; i1<2; i1++)
	    for (i2=0; i2<2; i2++)
	    for (i3=0; i3<2; i3++)
	      ascrule[2*i0+i1][2*i2+i3]=
		'0'+2*(evrule[i0][i1][i2][i3]-'0')+
		  (i1==0?bhrul1[i0][i2]:bhrul2[i0][i2])-'0';
	    xblnk();
	    rmenu();
	    break;
        case 'u':					/* sparse init arry */
	    xblnk();
	    for (i=0; i<AL; i++) arr1[i]=0;
	    arr1[AL/4]=1;
            arr1[AL/2]=1;
            arr1[(3*AL)/4]=1;
            arr1[(3*AL)/4+2]=1;
	    lmenu();
            break;
	case 'X':					/* random rule */
	    xblnk();
	    i=rand();
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	      if (i == 0) i=rand();
	      ascrule[i0][i1]='0'+i%KK;
	      i/=KK;
	    }}
	    rmenu();
	    break;
	case 'x':					/* random rule */
	    xblnk();
	    i=rand();
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	      if (i == 0) i=rand();
	      if (auxrule[i0][i1]==' ') {
		ascrule[i0][i1]='0'+i%KK;
		i/=KK;
		}
	      }}
	    rmenu();
	    break;
	case 'y':					/* random line */
	    xblnk();
	    ranlin();
            lmenu();
	    break;
	case 'z':
	    for (i=0; i<AL; i++) arr1[i]=0;
	    lmenu();
	    break;
	case 'Y':					/* symmetrize rule */
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    ascrule[i1][i0]=ascrule[i0][i1];      
	    }}
	    rmenu();
	    break;
	case 'B':					/* begin barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[0][a-'0']=a;
	    ascrule[1][a-'0']=a;
	    ascrule[2][a-'0']=a;
	    ascrule[3][a-'0']=a;
	    rmenu();
	    break;
	case 'E':					/* end barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[a-'0'][b-'0']=b;
	    ascrule[a-'0'][b-'0']=b;
	    ascrule[a-'0'][b-'0']=b;
	    ascrule[a-'0'][b-'0']=b;
	    rmenu();
	    break;
	case 'L':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    ascrule[a-'0'][b-'0']=c;
	    rmenu();
	    break;
	case 'R':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][b-'0']=a;
	    rmenu();
	    break;
	case 'S':					/* still life link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][b-'0']=b;
	    rmenu();
	    break;
	case 'U':					/* push rule */
	    if (rulptr<10) rulptr++;
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    rulstk[rulptr][i0][i1]=ascrule[i0][i1];
	    }}
	    xmenu(rulptr);
	    break;
	case 'V':					/* pop rule */
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    ascrule[i0][i1]=rulstk[rulptr][i0][i1];
	    }}
	    if (rulptr>0) rulptr--;
	    rmenu();
	    break;
	case 'G':					/* fetch rule */
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    ascrule[i0][i1]=rulstk[rulptr][i0][i1];
	    }}
	    rmenu();
	    break;
	case '=':
	    for (i=1; i<8;  i++) {
	    for (j=0; j<SL; j++) arr1[SL*i+j]=arr1[j];}
	    lmenu();
	    break;
	case '~':
	    for (i=1; i<16;  i++) {
	    for (j=0; j<20; j++) arr1[20*i+j]=arr1[j];}
	    lmenu();
	    break;
        default: break;
        }
    }
    if (more=='n') break;
    do {
    evolve();
    videocursor(0,0,0);
    scrstr("?");
    videocursor(0,0,34);
    scrstr("y/n/cr");
    more=kbdin();
    } while (more=='\015');
    videomode(T80X25);					/* reset the screen */
    if (more=='n') break;
    }
  videomode(T80X25);}    

/* list of demonstration rules */
int dmenu() {int ii, jj; char c;
videomode(T80X25);
videocursor(0,0,0); scrstr(" Demonstration Rules for LCAU4H");
videocursor(0,1,0); scrstr(" 1 -");
videocursor(0,2,0); scrstr(" 2 -");
videocursor(0,3,0); scrstr(" 3 - eventual glider gun");
videocursor(0,4,0); scrstr(" 4 - class iv");
videocursor(0,5,0); scrstr(" 5 - class iv");
videocursor(0,6,0); scrstr(" 6 - glider implosion");
videocursor(0,7,0); scrstr(" 7 - evolutionary #1");
videocursor(0,8,0); scrstr(" 8 - Fisch's eater");
videocursor(0,9,0); scrstr(" 9 - totalized eater");
videocursor(0,10,0); scrstr("10 - majority");
videocursor(0,11,0); scrstr("11 -");
videocursor(0,12,0); scrstr("12 -");
videocursor(0,13,0); scrstr("13 -");
videocursor(0,14,0); scrstr("14 -");
videocursor(0,15,0); scrstr("15 - reversible rule");
videocursor(0,16,0); scrstr("16 - big blotches");
videocursor(0,17,0); scrstr("17 - class iv on background");
videocursor(0,18,0); scrstr("18 - eater variant");
videocursor(0,19,0); scrstr("19 - pepper and salt");
videocursor(0,20,0); scrstr("20 - evolutionary #2");
videocursor(0,1,DCOLS); scrstr("21 ");
videocursor(0,2,DCOLS); scrstr("22 ");
videocursor(0,3,DCOLS); scrstr("23 ");
videocursor(0,4,DCOLS); scrstr("24 ");
videocursor(0,5,DCOLS); scrstr("25 ");
videocursor(0,6,DCOLS); scrstr("26 ");
videocursor(0,7,DCOLS); scrstr("27 ");
videocursor(0,8,DCOLS); scrstr("28 ");
videocursor(0,9,DCOLS); scrstr("29 ");
videocursor(0,10,DCOLS); scrstr("30 ");
videocursor(0,11,DCOLS); scrstr("31 ");
videocursor(0,12,DCOLS); scrstr("32 ");
videocursor(0,13,DCOLS); scrstr("33 ");
videocursor(0,14,DCOLS); scrstr("34 ");
videocursor(0,15,DCOLS); scrstr("35 ");
videocursor(0,16,DCOLS); scrstr("36 ");
videocursor(0,17,DCOLS); scrstr("37 ");
videocursor(0,18,DCOLS); scrstr("38 ");
videocursor(0,19,DCOLS); scrstr("39 ");
videocursor(0,20,DCOLS); scrstr("40 ");
videocursor(0,22,0); scrstr("move cursor with arrows, select with <cr>\n");
videocursor(0,23,0); scrstr("position at top with <, bottom with >.\n");
  ii=DROWS; jj=DCOLS; c='!'; /* ii = max # rows, jj = second column */
  do {
    videocursor(0,di+1,dj);
    while (!kbdst()) videocursor(0,di+1,dj);
    c=kbdin();
      switch (c) {
      case '\013': case '\310': if (di==0) di=ii-1; else di--; break; /* up arrow */
      case '\012': case '\320': if (di==ii-1) di=0; else di++; break; /* down arrow */
      case '\014': case '\315': case ' ': dj=(dj==1?jj:1); break;     /* forward */
      case '\010': case '\313': dj=(dj==1?jj:1); break;		      /* backward */
      case '<': di=0; break;					      /* home */
      case '>': di=ii-1; break;					      /* bottom */
      default: break;
      }
    } while (c!='\015');
  videomode(T80X25);
  return(di+(dj==1?0:ii)+1);
}

/* edit the rule */
edrule() {char c;
 while (0<1) {
    cscrrul();
    videocursor(0,3,ru);
    c=kbdin();
    if (c=='\015') break;				/* carriage return exits */
    switch (c) {
    case '0': case '1': case '2': case '3':		/* state */
      ascrule[ru0][ru1]=c;
      videocattr(0,c,3,1);
      rupl();
      break;
    case '\011':					/* tab = next quad */
      ru1=0; ru0++;
      if (ru0==KK) ru0=0;
      ru=KK*ru0+6;
      break;
    case ' ': case '\315': rupl(); break;				 /* advance */
    case '\010': case '\313': rumi(); break;				 /* backspace */
    case '\013': case '\310': auxrule[ru0][ru1]='+'; rupl(); break; /* up arrow */
    case '\012': case '\320': auxrule[ru0][ru1]=' '; rupl(); break; /* down arrow */
    case 'z': asczero();
    default: break;
    }
  }
}

/* advance the rule cursor */
rupl() {
  ru1++;
 if (ru1==KK) {ru1=0; ru0++;}
  if (ru0==KK) {ru0=0; }
  ru=KK*ru0+ru1+6;
  }

/* retract the rule cursor */
rumi() {
 if (ru1!=0) ru1--; else {ru1=KK-1;
  if (ru0!=0) ru0--; else {ru0=KK-1;
     }}
  ru=KK*ru0+ru1+6;
  }

/* edit totalistic rule */
edtrule() {char c; int  i, k;
  i=k=0;
  while (i<TS) {
    c=ttrule[i];
    videocursor(0,TRR,TRC+i);
    videocattr(0,c,3,1);
    c=kbdin();
    if (c=='\015') break;
    switch (c) {
      case '0': case '1': case '2': case '3':		/* state */
        ttrule[i]=c;
        videocattr(0,c,2,1);
        i++; break;
      case ' ': case '\315':				/* space = advance */
        i++; break;
      case '\010': case '\313':				/* backspace */
        if (i!=0) i--;
        break;
      case 'x':						/* random selection */
        for (i=0; i<TS; i++) {
          if (i%8==0) k=rand();
          ttrule[i]='0'+k%KK; k/=KK;}
        i=0;
        tscrrul();
        break;
      default: break;
      }
    }
}

/* edit the line of cells */
edline(nr,nc) int nr, nc; {
char c;
int  i, j, k, l, m;
int  aa, bb;
int  xx, yy;
int  ll;

  m=0; /* refresh trigger */
  cscrrul();
  videocursor(0,19,0);
  scrstr("insert states with 0, 1, 2, 3; move cursor");
  videocursor(0,20,0);
  scrstr("with keyboard arrows, space or backspace.");
  videocursor(0,21,0);
  scrstr("carriage return exits");
  while (0<1) {
  ll=lj;
  videocursor(0,lj+LROW,li);
  c=kbdin();
  if (c == '\015') {videoscroll(19,0,24,70,0,3); break;}
  switch (c) {
  case '0': case '1': case '2': case '3':		/* enter state */
    arr1[nc*lj+li]=c-'0';
    li++; if (li>=nc) {li=0; lj++;} if (lj>=nr) lj=0;
    break;
  case '\012': case '\320':				/* downwards */
    lj++; if (lj>=nr) lj=0;
    break;
  case '\013': case '\310':				/* upwards */
    if (lj>0) lj--; else lj=nr-1;
    break;
  case '\014': case '\315': case ' ':			/* forward */
    li++; if (li>=nc) {li=0; lj++;} if (lj>=nr) lj=0;
    break;
  case '\010': case '\313':				/* backward */
    if (li>0) li--;
      else {li=nc-1; if (lj>0) lj--; else lj=nr-1;}
    break;
  case '<':						/* left margin */
    li=0;
    break;
  case '>':						/* right margin */
    li=nc-1;
    break;
  case 'z':						/* clear single row */
    for (k=0; k<nc; k++) arr1[nc*lj+k]=0;
    break;
  case 'Z':						/* clear full array */
    for (k=0; k<AL; k++) arr1[k]=0;
    lmenu();
    break;
  case '^':						/* copy line above */
    for (k=0; k<nc; k++)
      arr1[nc*lj+k]=arr1[nc*(lj-1)+k];
    break;
  case 'x':						/* unmark transitions */
    auxblnk();
    cscrrul();
    break;				
  case 'q':						/* clean up the area */
    ulmenu(3);
    break;
  case '=':		/* insert point transition in ascrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    c=arr1[yy+nc]+'0';
    ascrule[arr1[xx]][arr1[yy]]=c;
    rmenu(); 
    cscrrul();
    break;
  case '+':		/* set marker for transition in auxrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    auxrule[arr1[xx]][arr1[yy]]='+';
    rmenu(); 
    cscrrul();
    break;
  case '-':		/* remove marker for transition from auxrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    auxrule[arr1[xx]][arr1[yy]]=' ';
    rmenu(); 
    cscrrul();
    break;
  case '*':		/* insert transitions for whole line */
    if (lj==0) break;
    for (i=0; i<nc; i++) {
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      c=arr1[yy+nc]+'0';
      ascrule[arr1[xx]][arr1[yy]]=c;
      auxrule[arr1[xx]][arr1[yy]]=c;
      }
    rmenu(); 
    cscrrul();
    break;
  case '.':		/* point evolution */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    arr1[yy+nc]=ascrule[arr1[xx]][arr1[yy]]-'0';
    break;
  case '?':		/* evolution of whole line */
    if (lj==nr-1) {for (i=0; i<AL-nc; i++) arr1[i]=arr1[i+nc]; lj--; lmenu(); }
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      arr1[yy+nc]=ascrule[arr1[xx]][arr1[yy]]-'0';
      }
    break;
  case '/':		/* conditional evolution */
    if (lj==nr-1) {for (k=0; k<AL-nc; k++) arr1[k]=arr1[k+nc]; lj--; lmenu(); }
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      j=0;
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      aa=arr1[xx]; if (aa>KK-1) j=KK;
      bb=arr1[yy]; if (bb>KK-1) j=KK;
      if (auxrule[aa][bb]==' ') j=KK;
      if (j==0) j=ascrule[aa][bb]-'0';
      arr1[yy+nc]=j;
      }
    break;
  case 'c':		/* test consistency everywhere */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr;  j++) {
    for (i=0; i<nc; i++) {
      yy=i+nc*(j-1);
      xx=i==0?yy+nc-1:yy-1;
      aa=arr1[xx];
      bb=arr1[yy];
      c=arr1[yy+nc]+'0';
      videocursor(0,j+LROW,i);
      videocattr(0,c=='0'?'.':c,c==ascrule[aa][bb]?7:4,1);
      }}
    m=1;
    break;
  case 'C':		/* test consistency for marked neighborhoods */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr;  j++) {
    for (i=0; i<nc; i++) {
      yy=i+nc*(j-1);
      xx=i==0?yy+nc-1:yy-1;
      aa=arr1[xx];
      bb=arr1[yy];
      c=arr1[yy+nc]+'0';
      videocursor(0,j+LROW,i);
      videocattr(0,c=='0'?'.':c,
        (c==ascrule[aa][bb] && auxrule[aa][bb]!=' ')?7:4,1);
      }}
    m=1;
    break;
  default: break;
      } /* end switch */
  if (m==0) for (k=0; k<nc; k++) {
    videocursor(0,ll+LROW,k);
    l=arr1[nc*ll+k]+'0';
    videoputc(l=='0'?'.':l,1);
    }
  m=0;
  }     /* end while */
}       /* end edline */

/* calculate index, or rather the neighbors, in edline. */
linix(xx,i,j,nc) int xx[RR+1], i, j, nc; {int k, x;
  xx[(RR-1)/2]=x=i+nc*(j-1);
if (i==0) xx[0]=x+nc-1; else xx[0]=x-1;
  if (i==nc-1) xx[2]=x-nc+1; else xx[2]=x+1;
  for (k=0; k<RR; k++) xx[k]=arr1[xx[k]];
  xx[RR]=arr1[x+nc];
}

/* display a screen of evolution */
evolve() {int i, j;
//  if (sc==7) {xevolve(); return;}
asctobin();
  for (j=0; j<200; j++) {				/* evolve for 192 generations */
    for (i=0; i<AL; i++) videodot(j,i,arr1[i]);
    onegen(AL);
    }
}

/* display a screen of evolution in Plantronics mode*/
xevolve() {int i, j, k, o, p; char l, m, n;
//  videomode(COLGRAF);					/* erase the screen */
//  outportb(0x3dd,0x20);
//  videocursor(0,0,0);					/* top text line */
//  scrstr(":");
//  scrrul();
 asctobin();
  for (j=0; j<200; j++)					/* evolve for 192 generations */
    {
    for (i=0; i<AL/4; i++)
      {
      for (k=0, m=0, n=0; k<4; k++)
        {
	p=2*(4*i+k+AL);
        l=arr1[((p-j)/2)%AL];
        m=(m<<1)|(l&0x1); l>>=1; n=(n<<1)|(l&0x1);
        l=arr1[((p+1-j)/2)%AL];
        m=(m<<1)|(l&0x1); l>>=1; n=(n<<1)|(l&0x1);
        }
      o=80*(j/2)+i;
//      if (j%2==0) pokeb(0xb800,o,m); else pokeb(0xba00,o,m);
//      if (j%2==0) pokeb(0xbc00,o,n); else pokeb(0xbe00,o,n);
      }
    onegen(AL);
//    if (kbdst()) {kbdin(); break;}
    }
}

/* copy ascrule over to binrul */
asctobin() {int i0, i1;
for (i0=0; i0<KK; i0++) {
  for (i1=0; i1<KK; i1++) {
    binrule[i0][i1]=ascrule[i0][i1]-'0';
    }}
}

/* evolution for one generation */
onegen(j) int j; {int i;
 if (j==0) return;
    else if (j==1) {arr1[0]=binrule[arr1[0]][arr1[0]]; return;}
    else {arr2[0]=binrule[arr1[j-1]][arr1[0]];
       for (i=1; i<j; i++) arr2[i]=binrule[arr1[i-1]][arr1[i]];
       for (i=0; i<j; i++) arr1[i]=arr2[i];}
}

/* uniform line menu */
ulmenu(l) int l; {int i, j, k;
 for (j=0; j*SL<AL; j++) {
  for (i=0; i<SL;    i++) {
    videocursor(0,LROW+j,i);
    k=arr1[j*SL+i];
    videocattr(0,k==0?'.':k+'0',l,1);
    }}
  }

/* map the automaton state into the unit square */
pprob() {int i, j; float x, y, z, k, kay;
i=AL/2; kay=1.0/(float)KK; z=(float)((128/(KK*KK))*KK*KK);
x=0.0; k=kay; for (j=0; j<20; j++, k*=kay) x+=k*(float)arr1[i+j];
y=0.0; k=kay; for (j=1; j<20; j++, k*=kay) y+=k*(float)arr1[i-j];
videodot((int)(z*(1.0-y)),(int)(z*x),1);
}

/* set up a grid for the unit square */
pgrid() {int i, j, k, l;
k=KK*KK+1;
l=195/(k-1);
for (i=0; i<k; i++) {
for (j=0; j<k; j++) {
videodot(l*i,l*j,2);
}}
}

/* tutorial and Help screen */
tuto() {
    videomode(T80X25);
 videocursor(0,2,0);
    scrstr("<Copyright (C) 1987, 1988, 1989 - H.V.McIntosh, G.Cisneros S.>");
    videocursor(0,4,0);
    scrstr("             ** LIFE in One Dimension **");
    videocursor(0,6,0);
    scrstr("Four States - Black(0), Cyan(1), Magenta(2), White(3).");
    videocursor(0,8,0);
    scrstr("Leftsided neighbors - two cells per neighborhood.");
    videocursor(0,10,0);
    scrstr("Complete transition rule - random, edited, or stored.");
    videocursor(0,12,0);
    scrstr("Totalistic transition rule - random, edited, or stored.");
    videocursor(0,14,0);
    scrstr("Initial Cellular Array - random, edited, or patterned.");
    videocursor(0,16,0);
    scrstr("Submenus in options t(probabilities) and d(de Bruijn)");
    videocursor(0,17,0);
    scrstr("will be displayed in response to typing ?");
    videocursor(0,19,0);
    scrstr("Use any key to terminate a display in progress.");
    videocursor(0,22,0);
    scrstr("now, ... press any key to continue.");
}

/* rule menu */
rmenu() {
 videocursor(0,1,0);
    scrstr("      0000111122223333");
    videocursor(0,2,0);
    scrstr("      0123012301230123");
    videocursor(0,3,0);
    scrstr("Rule: ");
    scrrul();
    if (istot()==1) {videocursor(0,TRR,TRC); xmenu(totonu(0));}
    videocursor(0,5,0);
    scrstr("    r(rule), l(line), #(stored), g(graph), q(quit), t(prob),");
    videocursor(0,6,0);
    scrstr("           x(rand rule), y(rand line), u(unit line), d(deBruijn)");
    videocursor(0,7,0);
    scrstr("           @nn(tot/rule), $nn(12 tot/rules), T(ed tot/rule).");
    videocursor(0,5,0);
    }

/* totalistic rule menu*/
tmenu() {
 videocursor(0,TRR-2,TRC-6);
    scrstr("      totalistic rule");
    videocursor(0,TRR-1,TRC-6);
    scrstr("      0.1.2.3");
    videocursor(0,TRR,TRC-6);
    scrstr("rule: ");
    tscrrul();
    videocursor(0,TRR,TRC);
    }

/* plain line menu */
lmenu() {int i, j; char c;
  for (j=0; SL*j<AL; j++) {videocursor(0,LROW+j,0);
  for (i=0; i<SL; i++) {c='0'+arr1[SL*j+i]; videoputc(c=='0'?'.':c,1); } }
  videocursor(0,5,0); }

/* color line menu */
clmenu() {int c, i, j;
 for (j=0; SL*j<AL; j++) {
    for (i=0; i<SL; i++) {
      videocursor(0,LROW+j,i);
      c=arr1[SL*j+i];
      videocattr(0,c==0?'.':'0'+c,color(c),1);
      }}
  videocursor(0,5,0); }

/* get compatible color */
int color(i) int i; {
 switch (i) {
    case 0:  return 2;
    case 1:  return 3;
    case 2:  return 5;
    case 3:  return 7;
    default: return 6;
}}

/* display rule number */
xmenu(n) int n; {
char xx[6]; int i, nn;  
  nn=sprintf(xx,"%5u",n);
  videocursor(0,XRR,XRC);
  for (i=0; i<nn; i++) videoputc(xx[i],1);
  videocursor(0,XRR,XRC); }

/* clear screen space for rule number */
xblnk() {videocursor(0,XRR,XRC); scrstr("     "); videocursor(0,5,0); }

/* copy saved rule #n into active rule */
xtoasc(n) int n; {int i0, i1;
xmenu(n+1);
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
  ascrule[i0][i1]=ixrule[n][i0*KK+i1];
}

/* change totalistic rule to general rule */
totoasc() {int i0, i1;
for (i0=0; i0<KK; i0++)
     for (i1=0; i1<KK; i1++){
          ascrule[i0][i1]=ttrule[i0+i1];
           }
}

/* change decimal totalistic rule to sum values */
nutoto(x) int x; {int i;
for (i=0; i<TS; i++) {ttrule[i]=x%KK+'0'; x/=KK;}}

/* change sum values to decimal totalistic rule */
int totonu(i) int i; {int r;
if (i<TS) r=(ttrule[i]-'0')+KK*totonu(i+1); else r=0;
 return r; }

/* test whether a rule is totalistic */
int istot() {char trule[TS]; int i0, i1, l;
    l=1;
    trule[0]=ascrule[0][0];
    trule[1]=ascrule[1][0];
    trule[2]=ascrule[2][0];
    trule[3]=ascrule[3][0];
    trule[4]=ascrule[3][1];
    trule[5]=ascrule[3][2];
    trule[6]=ascrule[3][3];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++) {
    if (ascrule[i0][i1]!=trule[i0+i1]) l=0;
    }
    return l; }

/* find the totalistic rule number of a known ascii totalistic rule */
void asctotot() {int i0, i1;
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
      ttrule[i0+i1] = ascrule[i0][i1];
}

/* generate a random rule */
ranrul() {int i, i0, i1;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++) {
  i=(random()&0xFF)%KK;
  ascrule[i0][i1]=i+'0';
  }
}

/* generate a random line of cells in arr1 */
ranlin() {int i, c; for (i=0; i<AL; i++) arr1[i]=(random())%KK;}

/* put a single i-cell on a line of j-cells */
void putionj(i,j) int i, j; 
{int k; for (k=0; k<AL; k++) arr1[k]=j; arr1[AL/2]=i;}

/* clear auxrule to blanks */
auxblnk() {int i0, i1;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
  auxrule[i0][i1]=' ';
}

/* clear ascrule to zeroes */
asczero() {int i0, i1;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
  ascrule[i0][i1]='0';
}

/* limit j to interval (i,k) */
int lim(i,j,k) int i, j, k; {if (i>=j) return i; if (k<=j) return k; return j;}

/* report Wolfram rule number or variant */
unsigned long int wolnum(c) char c; {
unsigned long int  k;
int       i0, i1;
 k=0;
  switch (c) {
    case 'w':
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=KK*k+(ascrule[i0][i1]-'0');
      break;
    case 'c':
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=KK*k+((ascrule[(i0+1)%KK][(i1+1)%KK]-'0')+1)%KK;
      break;
    case 'r':
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=KK*k+(ascrule[i1][i0]-'0');
      break;
    case 'b':
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=KK*k+((ascrule[(i1+1)%KK][(i0+1)%KK]-'0')+1)%KK;
      break;
    default: k=0; break;}
        return k;
}

/* display the rule number on the screen */
scrrul() {int i0, i1;
for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  videoputc(ascrule[i0][i1],1);
  }

/* display the rule number on the screen in glorious technicolor */
cscrrul() {int i, i0, i1;
  i=6;
for (i0=0; i0<KK; i0++) {
  for (i1=0; i1<KK; i1++) {
  videocursor(0,3,i++);
  videocattr(0,ascrule[i0][i1],auxrule[i0][i1]==' '?3:6,1);
  }} }

/* display totalistic rule number by sum */
tscrrul() {int i;
 for (i=0; i<TS; i++) {videoputc(ttrule[i],1); } }

/* display the hexadecimal rule number on the screen */
hscrrul() {char i; int i0, i1;
for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1+=2)
    i=ascrule[i0][i1]+4*(ascrule[i0][i1+1]-'0');
  if (i>'9') i=i+('A'-':');
  videoputc(i,1);      
  }

/* change decimal totalistic rule to sum values */
/* read a decimal number and put 16 bits in evrule */
nutoevo(x) unsigned int x; {int i0,i1,i2,i3;
for (i0=0; i0<2; i0++)
for (i1=0; i1<2; i1++)
for (i2=0; i2<2; i2++)
for (i3=0; i3<2; i3++)
  {evrule[i0][i1][i2][i3]=x%2+'0'; x/=2;}}

/* change decimal totalistic rule to sum values */
/* read a decimal number and put 4 bits in bhrul1 */
nutobh1(x) int x; {int i0, i1;
for (i0=0; i0<2; i0++)
for (i1=0; i1<2; i1++)
  {bhrul1[i0][i1]=x%2+'0'; x/=2;}}

/* change decimal totalistic rule to sum values */
/* read a decimal number and put 4 bits in bhrul2 */
nutobh2(x) int x; {int i0, i1;
for (i0=0; i0<2; i0++)
for (i1=0; i1<2; i1++)
  {bhrul2[i0][i1]=x%2+'0'; x/=2;}}

/* change decimal totalistic rule to sum values */
/* read a decimal number and put 4 bits in prule1 */
nutowo1(x) int x; {int i;
for (i=0; i<4; i++) {prule1[i]=x%2+'0'; x/=2;}}

/* change Wolfram rule number to decimal rule number */
int wotonu1(i) int i; {int r;
if (i<4) r=(prule1[i]-'0')+2*totonu(i+1); else r=0;
  return r; }

/* change decimal number to Wolfram rule number */
/* read a decimal number and put 4 bits in prule2 */
nutowo2(x) int x; {int i;
for (i=0; i<4; i++) {prule2[i]=x%2+'0'; x/=2;}}

/* change sum values to decimal totalistic rule */
int wotonu2(i) int i; {int r;
if (i<4) r=(prule2[i]-'0')+2*totonu(i+1); else r=0;
  return r; }

/* turn ascrule into its Wolfram number and place on screen at (i,j) */
woruno(i,j) int i, j; {
char xx[10];
int  k, l, i0, i1;
  k=0;
for (i0=KK-1; i0>=0; i0--)
  for (i1=KK-1; i1>=0; i1--)
    k=3*k+(ascrule[i0][i1]-'0');
  k=sprintf(xx,"%3d",k);
  videocursor(0,i,j);
  for (l=0; l<k; l++) videoputc(xx[l],1);
}

/* write a string in graphics mode */
scrstr(s) char *s; {for (; *s != '\0'; s++) videoputc(*s,1); }

/* keyboard status */
int kbdst() {return(bdos(11) & 0xFF);}

/* direct keyboard input, no echo */
kbdin() {int c;
  if ((c = bdos(7) & 0xFF) == '\0') c = (bdos(7) & 0xFF) | 0x80;
  return(c);
}

/* read number from keyboard */
int numin(n) int n; {char c;
  c=kbdin();
  if (c>='0'&&c<='9') return(numin(10*n+(c-'0'))); else return(n);
}

void rule4h(){
  ranrul();
    xtoasc(((int)random()%NX));
    ranlin();					/* random initial array */
}

void savearry() {int i; for (i=0; i<AL; i++) arr3[i]=arr1[i];}
void renewarry() {int i; for (i=0; i<AL; i++) arr1[i]=arr3[i];}

int  bdos(){}

/************************  modify added  *******************************/

/* clear ttrule to zeroes */
ttzero() 
  {
    int i;
    for (i=0; i<TS; i++)  ttrule[i]='0';
   }

/* clear strule to zeroes */
stzero() 
{
int i, j;
  for (i=0; i<KK; i++)
      for (j=0; j<SS; j++)  strule[i][j]='0';
 }

/* clear gf4rule to zeroes */
gfzero() {
 int i;
    for (i=0; i<13; i++) gf4rule[i]='0';
   }

/* clear auxrule to zeroes */
auxzero()
{
int i0, i1, i2;
  for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
       for (i2=0; i2<KK; i2++)
            auxrule[i0][i2]=0;
}

/* test whether a rule is semitotalistic */
int isstot()
 { 
   int i0, i1, i2, l, rule[KK][5];
    l=1;
    rule[0][0]=ascrule[0][0];
    rule[0][1]=ascrule[1][0];
    rule[0][2]=ascrule[2][0];
    rule[0][3]=ascrule[2][1];
    rule[0][4]=ascrule[2][2];
    rule[1][0]=ascrule[0][0];
    rule[1][1]=ascrule[1][0];
    rule[1][2]=ascrule[2][0];
    rule[1][3]=ascrule[2][1];
    rule[1][4]=ascrule[2][2];
    rule[2][0]=ascrule[0][0];
    rule[2][1]=ascrule[1][0];
    rule[2][2]=ascrule[2][0];
    rule[2][3]=ascrule[2][1];
    rule[2][4]=ascrule[2][2];
    for (i0=0; i0<KK; i0++)
      for (i1=0; i1<KK; i1++)
          for (i2=0; i2<KK; i2++)
                if (ascrule[i0][i2]!=rule[i1][i0+i2]) l=0;
    return l;
  }
/* generate a random line of cells in arr1 with assigned probabilities */
ranblin(p1,p2,p3,p4) double p1, p2, p3, p4; {
int    i, c, i1, i2, i3;
double p;
  p=p1+p2+p3+p4;
  i1=(int)((1000.0*p1)/p);
  i2=i1+(int)((1000.0*p2)/p);
  i3=i1+i2+(int)((1000.0*p3)/p);
  for (i=0; i<AL; i++) {
       c=random()%1000;
        if (c<i1) arr1[i]=0;
        else if (c<i2) arr1[i]=1; 
        else if (c<i3) arr1[i]=2; 
             else arr1[i]=3;
       }
}

/* change a GF4 rule to general rule */
void gftoasc() 
{
  int i0, i1, i2;
for (i0=0; i0<KK; i0++)
     for (i1=0; i1<KK; i1++)
          for (i2=0; i2<KK; i2++) 
               ascrule[i0][i2]=
               (i0*((gf4rule[0]-'0')*i0+(gf4rule[9]-'0'))+
                i0*(gf4rule[1]-'0')*i1+
                i0*(gf4rule[2]-'0')*i2+
                i1*((gf4rule[3]-'0')*i0+(gf4rule[10]-'0'))+
                i1*(gf4rule[4]-'0')*i1+
                i1*(gf4rule[5]-'0')*i2+
                i2*((gf4rule[6]-'0')*i0+(gf4rule[11]-'0'))+
                i2*(gf4rule[7]-'0')*i1+
                i2*(gf4rule[8]-'0')*i2+
                 (gf4rule[12]-'0') )%KK+'0';
}

/* change semitotalistic rule to general rule */
sttoasc() 
{
int i0, i1, i2;
for (i0=0; i0<KK; i0++)
     for (i1=0; i1<KK; i1++)
          for (i2=0; i2<KK; i2++)
               ascrule[i0][i2]=strule[i1][i0+i2];
}

/* end nxlcau4h.c */
