/* rectbl.h -- G. Cisneros, 4.91; rev. 2.10.91 */
/* Mandatory declarations */

# include "rec.h"

int r_lpar();		/* compilation of a left parenthesis 		*/
int r_rpar();		/* compilation of a right parenthesis 		*/
int r_colon();		/* compilation of a colon 			*/
int r_semicol();	/* compilation of a semicolon 			*/
int r_code();		/* compilation of a simple operator 		*/
int r_oper1();		/* compilation of an operator with an ASCII arg */
int r_pred();		/* compilation of a simple predicate		*/
int r_pred1();		/* compilation of a predicate with an ASCII arg */
int r_noopc();		/* compilation of no-op 			*/
int r_comment();	/* compilation of a (bracket enclosed) comment 	*/
int r_ubrack();		/* compilation of an unbalanced right bracket 	*/
int r_lbrace();		/* compilation of a (brace enclosed) program 	*/
int r_ubrace();		/* compilation of an unbalanced right brace 	*/

int r_call();		/* execution of predicate @x (call subr. x) 	*/
int r_quit();		/* execution of operator _ (exit to O.S.) 	*/
int r_xbrace();		/* execution of brace-enclosed program 		*/

/* Optional: Counter predicate */

 int r_ctrc();		/* compilation of a counter */
 int r_ctrx();		/* execution of a counter */

/* Optional declarations, needed only in versions using quotes and
   numbers (see cnum.c and cquo.c); r_ld, r_ldch and r_xstr must
   be provided by user */

int r_dquote(); /* compile doubly quoted string into symbol table */ 
int r_squote(); /* compile singly quoted char into the program array */
int r_cmin();   /* compile '-' as simple operator, call r_cnum if followed by digit or period */                

int r_cnum();   /* compile a number (WORD, LONG or REAL) into sym. table */

int r_ld();	/* execution of compiled numbers */
int r_ldch();	/* execution of apostrophe (single quote op.) */
int r_xstr();	/* execution of quoted strings */

int r_cdblp(), r_ldblp();
int r_cstrp();
int r_lstrp();
int ropuw();
/* Declarations of user-provided execution subroutines */

int rop0(), rop1(), rop2(),rop3(), roplz();
// int ropuw();
int roppc(), roplr(), ropll();


/* ............................................*/

/* Table of compiling/executing function definitions,
   a pair for each printing ASCII character */

struct fptbl dtbl[] = {
	r_noopc,	False,	" ",	/* [blank] */
	r_ctrc,		r_ctrx,	"count to n  ",  /* ! counter !n!, n decimal */
        r_cstrp,        r_lstrp,"\" - quoted string",   /* here is */
        r_noopc,        False,  " ",    /* # */
        r_noopc,        False,  " ",    /* $ */
        r_pred,         roppc,  "%  - 50% T predicate ",    /* % */
	r_noopc,	False,	" ",	/* & */
	r_noopc,	False,	" ",	/* ' quoted char */
	r_lpar,		False,	" ",	/* ( start of expression */
	r_rpar,		False,	" ",	/* ) end of expression */
	r_noopc,	False,	" ",	/* * */
	r_noopc,	False,	" ",	/* + */
	r_noopc,	False,	" ",	/* , separator */
	r_noopc,	False,	" ",	/* - */
	r_noopc,	False,	" ",	/* . f.p. number  */
        r_noopc,        False,  " ",    /* / */
 	r_code,		rop0,   "0 - set state 0",	/* 0 number */
	r_code,		rop1,   "1 - set state 1",	/* 1 number */
	r_code,  	rop2,   "2 - set state 2",	/* 2 number */
	r_code,		rop3,   "3 - set state 3",	/* 3 number */
	r_noopc,	False,	" ",			/* 4 number */	
	r_noopc,	False,	" ",			/* 5 number */
	r_noopc,	False,	" ",			/* 6 number */
	r_noopc,	False,	" ",			/* 7 number */
	r_noopc,	False,	" ",			/* 8 number */
	r_noopc,	False,	" ",			/* 9 number */	
        r_colon,	False,	"repeat from opening lparen ",				/* : iteration */
	r_semicol,	False,	"exit at terminating rparen with the value T ", 	/* ; True termination */
	r_noopc,	False,	" ",							/* < */
	r_noopc,	False,	" ",							/* = */
	r_noopc,	False,	" ",							/* > */
	r_noopc,        False,  " ",                                                    /* ? */
	r_pred1,	r_call,	"call a subroutine, as in @x to call x",	/* @ call a subroutine */
	r_noopc,	False,	" ",							/* A */
	r_noopc,	False,	" ",							/* B */
	r_noopc,	False,	" ",							/* C */
	r_noopc,	False,	" ",							/* D */
	r_noopc,	False,	" ",							/* E */
	r_noopc,	False,	" ",							/* F */
        r_noopc,	False,	" ",							/* G */
	r_noopc,	False,	" ",							/* H */
	r_noopc,	False,	" ",							/* I */
	r_noopc,	False,	" ",							/* J */
	r_noopc,	False,	" ",							/* K */
	r_noopc,	False,	" ",							/* L */
	r_noopc,	False,	" ",							/* M */
	r_noopc,	False,	" ",							/* N */
	r_noopc,	False,	" ",							/* O */
	r_noopc,	False,	" ",							/* P */
        r_noopc,	False,	" ",							/* Q */
	r_noopc,	False,	" ",							/* R */
	r_noopc,	False,	" ",							/* S */
	r_noopc,	False,	" ",							/* T */
	r_noopc,	False,	" ",							/* U */
	r_noopc,	False,	" ",							/* V */
	r_code, 	ropuw,	"W - Wolfram rule mumber ",				/* W */
	r_noopc,	False,	" ",							/* X */
	r_noopc,	False,	" ",							/* Y */
	r_noopc,	False,	" ",							/* Z */
	r_comment,	False,	"begin comment",					/* [ start comment */
	r_noopc,	False,	" ",							/* \ */
	r_ubrack,	False,	"end comment   ",			/* ] end comment, otherwise error */
	r_noopc,	False,	" ",							/* ^ */
	r_code,		r_quit,	" ",							/* _ panic exit */
	r_noopc,	False,	" ",							/* ` */
	r_noopc,	False,	" ",							/* a */
	r_noopc,	False,	" ",							/* b */
	r_noopc,	False,	" ",							/* c */
	r_noopc,	False,	" ",							/* d */
	r_noopc,	False,	" ",							/* e */
        r_noopc,	False,	" ",							/* f */
	r_noopc,	False,	" ",							/* g */
	r_noopc,	False,	" ",							/* h */
	r_noopc,	False,	" ",							/* i */
	r_noopc,	False,	" ",							/* j */
	r_noopc,	False,	" ",							/* k */
	r_pred, 	ropll,  "l -  move cursor left if possible",			/* l */
	r_noopc,	False,	" ",							/* m */
	r_noopc,	False,	" ",							/* n */
	r_noopc,	False,	" ",							/* o */
	r_noopc,	False,	" ",							/* p */
	r_noopc,	False,	" ",							/* q */
 	r_pred, 	roplr,  "r -  move cursor rigth if possible",			/* r */
	r_noopc,	False,	" ",							/* s */
	r_noopc,	False,	" ",							/* t */
	r_noopc,	False,	" ",							/* u */
	r_noopc,	False,	" ",							/* v */
	r_noopc,	False,	" ",							/* w */
	r_noopc,	False,	" ",							/* x */
	r_noopc,	False,	" ",							/* y */
	r_code,   	roplz,  "z - bring cursor to origin",				/* z */
	r_lbrace,	r_xbrace,  " ",							/* { start of program */
	r_noopc,	False,	" ",							/* | */
	r_ubrace,	False,	" ",						/* } end of prog, else error */
	r_noopc,	False,	" "							/* ~ */
};
